//includes
var express = require('express');
var wpi = require('wiring-pi');	
var fs = require('fs');

//initialisation du serveur web, des chemins locaux et du socket
var app = express();
app.engine('.html',require('ejs').__express);
app.set('views',__dirname + '/templates');
app.use('/jquery',express.static(__dirname + '/node_modules/jquery/dist/'));
app.use('/socketio',express.static(__dirname + '/node_modules/socket.io-client/dist/'));
app.use('/public',express.static(__dirname +  '/public/'));
var httpserver = require('http').createServer(app);
var socketio = require('socket.io').listen(httpserver);

//numerotage des 'pin' suivant GPIO
wpi.setup('gpio');

//lecture du fichier de config
var config = JSON.parse(fs.readFileSync('configEssais.json'));
//console.log(config);
//initialisation des périphériques
for (var i = 0;i < config.perif.length;i++)
{   
	wpi.pinMode(config.perif[i].ouv,wpi.OUTPUT);
	wpi.pinMode(config.perif[i].fer,wpi.OUTPUT);
	wpi.pullUpDnControl(config.perif[i].bp,wpi.PUD_UP);	
	//positionement de la property etat du perif
	//config.perif[i].etat= wpi.digitalRead(config.perif[i].out);
	//abonnement aux interupt. front descendant sur les inputs
	wpi.wiringPiISR(config.perif[i].bp,wpi.INT_EDGE_FALLING,function()
	{
		for (var i = 0;i < config.perif.length;i++)
		{//recherche le l'input à low
			if (wpi.digitalRead(config.perif[i].bp) == 0)
			{
				// délai antirebond du bp
				wpi.delay(400);
				//appel de changement d'etat de l'output
				changeGPIO(config.perif[i].iden);					
			}
		}
	});				
}
			
//Route de distribution de la page web par defaut
app.get('/',function(req,res){
	res.render('indexEssais1.html');
});

//Route qui retourner les paramètres et leurs état 
//au format Json (pas utile ici, mais c'est cadeau)
//app.get('/perif',function(req,res){
//		res.json(config.perif);
//});

//socketio à ecoute
socketio.sockets.on('connection',function(client){
	 
	 //fourni la configuration au client qui se connecte
	 client.emit('connected',{config : config}); 	 
	 
	 //recoit une demande de changement du client	
	 client.on('perifchange',function(data)
	 {	 
		 //appel de changement d'etat de l'output
		 changeGPIO(data.iden);		 		 
	 });
});

//change l'etat de la sortie du perif i
function changeGPIO(i)
{
	var t = config.perif[i].taux;
	var e = config.perif[i].etat;
	var p = config.perif[i].position;
	var dep = config.perif[i].dep;
	var s = config.perif[i].sens;
	var temp = config.perif[i].tempo;
	function fermeture()
		{
			//calcul du taux de progression
			temp = config.perif[i].tempo;
			t = Math.ceil((p/temp)*100);
			dep = config.perif[i].dep;
			p ++;
			//compteur ++;
			//met a jour la property etat et taux du perif
			config.perif[i].etat = e;
			config.perif[i].taux = t;
			config.perif[i].position = p;
			//broadcast le status du perif à tous les clients
			var data = {'etat' : e,'iden' : i,'taux' : t};
			socketio.sockets.emit('perifchange',data);
			if ((p > temp) || (dep == 55))
			{
				clearInterval(x);
				//wpi.digitalWrite(config.perif[i].ouv,0);
				//wpi.digitalWrite(config.perif[i].fer,0);
				if (dep == 55)
				{
					e = 3;
					config.perif[i].etat = e;
					data = {'etat' : e,'iden' : i,'taux' : t};
					socketio.sockets.emit('perifchange',data);
					wpi.digitalWrite(config.perif[i].ouv,0);
					wpi.digitalWrite(config.perif[i].fer,0);
					console.log(config.perif[i].name+" arrêt en cours de descente,position =  "+p+" dep = "+dep+ " taux = "+t+" sens = "+s);
				}
				if (p > temp){
					e = 4;
					p = temp;
					dep = 55;
					config.perif[i].etat = e;
					config.perif[i].position = p;
					config.perif[i].dep = dep;
					data = {'etat' : e,'iden' : i,'taux' : t};
					socketio.sockets.emit('perifchange',data);
					wpi.digitalWrite(config.perif[i].ouv,0);
					wpi.digitalWrite(config.perif[i].fer,0);
					console.log(config.perif[i].name+" arrêt en bas, position = "+p+" dep = "+dep+ " taux = "+t+" sens = "+s);
				}
			}
		}
	function ouverture()
		{
			//calcul du taux de progression

			temp = config.perif[i].tempo;
			t = Math.ceil(((temp - p)/temp)*100);
			dep = config.perif[i].dep;
			p --;
			//compteur --;
			//met à jour les property etat et taux du perif
			config.perif[i].etat = e;
			config.perif[i].taux = t;
			config.perif[i].position = p;
			//broadsast le status du perif à tous les clients
			var data = {'etat' : e,'iden' : i,'taux' : t};
			socketio.sockets.emit('perifchange',data);
			if ((p < 1) || (dep == 55))
			{
				clearInterval(x);
				//wpi.digitalWrite(config.perif[i].ouv,0);
				//wpi.digitalWrite(config.perif[i].fer,0);
				if (dep == 55){
					e = 5;
					config.perif[i].etat = e;
					data = {'etat' : e,'iden' : i,'taux' : t};
					socketio.sockets.emit('perifchange',data);
					wpi.digitalWrite(config.perif[i].ouv,0);
					wpi.digitalWrite(config.perif[i].fer,0);
					console.log(config.perif[i].name+" arrêt en cours de montée, position =  "+p+" dep = "+dep+ " taux = "+t+" sens = "+s);
				}
				if (p < 1){
					e = 1;
					p = 1;
					dep = 55;
					t = 0;
					config.perif[i].dep = dep;
					config.perif[i].etat = e;
					config.perif[i].position = p;
					config.perif[i].taux = t;
					data = {'etat' : e,'iden' : i,'taux' : t};
					socketio.sockets.emit('perifchange',data);
					wpi.digitalWrite(config.perif[i].ouv,0);
					wpi.digitalWrite(config.perif[i].fer,0);
					console.log(config.perif[i].name+" arrêt en haut, position = "+p+" dep = "+dep+ " taux = "+t+" sens = "+s);
				}
			}
		}

	if (dep == 55) // volet à l'arrêt
		{	
		if (p == 1) //volet ouvert donc en haut
			{
			dep = 33;
			e = 2;
			s = "descente";
			config.perif[i].sens = s;
			config.perif[i].dep = dep;
			config.perif[i].etat = e;
			data = {'etat' : e,'iden' : i,'taux' : t};
			socketio.sockets.emit('perifchange',data);
			var compteur =1;
			wpi.digitalWrite(config.perif[i].ouv,0);
			wpi.digitalWrite(config.perif[i].fer,1);
			var x = setInterval(fermeture,500);
			
			}
			else if(p == temp)//volet fermé donc en bas
				{
				dep = 33;
				e = 6;
				t = 0;
				s = "montee";
				config.perif[i].dep = dep;
				config.perif[i].etat = e;
				config.perif[i].taux = t;
				config.perif[i].sens = s;
				data = {'etat' : e,'iden' : i,'taux' :t};
				socketio.sockets.emit('perifchange',data);
				var compteur =50;
				wpi.digitalWrite(config.perif[i].ouv,1);
				wpi.digitalWrite(config.perif[i].fer,0);
				var x = setInterval(ouverture,500);
				}
				else if(s == "montee")//volet arrêté en position intermédiaire montée
					{
					dep = 33;
					e = 2;
					s = "descente";
					config.perif[i].dep = dep;
					config.perif[i].etat = e;
					config.perif[i].sens = s;
					data = {'etat' : e,'iden' : i,'taux' : t};
					socketio.sockets.emit('perifchange',data);
					var compteur = p;
					wpi.digitalWrite(config.perif[i].ouv,0);
					wpi.digitalWrite(config.perif[i].fer,1);
					var x = setInterval(fermeture, 500);
					}
					else{
						dep = 33;
						e = 6;
						s = "montee";
						config.perif[i].dep = dep;
						config.perif[i].etat = e;
						config.perif[i].sens = s;
						data = {'etat' : e,'iden' : i,'taux' : t};
						socketio.sockets.emit('perifchange',data);
						var compteur = p;
						wpi.digitalWrite(config.perif[i].ouv,1);
						wpi.digitalWrite(config.perif[i].fer,0);
						var x = setInterval(ouverture, 500);
					}

		}
	else if(dep == 33)//volet en mouvement
		{
			dep = 55;
			config.perif[i].dep = dep;
			wpi.digitalWrite(config.perif[i].ouv,0);
			wpi.digitalWrite(config.perif[i].fer,0);
		}
	
	
	//monitoring dans la console pour faire joli
	console.log(new Date().toLocaleString() + ' : ' + config.perif[i].name + ' is set to ' + e);
}	


//demarre le serveur web
httpserver.listen(8080);

console.log ('CTRL+C pour quitter');


